/************************************************************************
 *   IRC - Internet Relay Chat, src/match.c
 *   Copyright (C) 1990 Jarkko Oikarinen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: oldmatch.c,v 1.1.1.1 2002/10/13 12:37:36 newiceman Exp $
 *
 */
#include "irc_string.h"
#include "config.h"      /* RFC1035_ANAL */
#include "client.h"
#include <assert.h>
#include <string.h> /* strrchr */
#include <ctype.h>
#include <stdlib.h>
#include "ircd.h"

/* match()
 * 
 *  Compare if a given string (name) matches the given
 *  mask (which can contain wild cards: '*' - match any
 *  number of chars, '?' - match any single character.
 *
 *      return  1, if match
 *              0, if no match
 *
 *  Originally by Douglas A Lewis (dalewis@acsu.buffalo.edu)
 */
#define MATCH_MAX_CALLS 512  /* ACK! This dies when it's less that this
                                and we have long lines to parse */
int match(const char *mask, const char *name)
{
  const unsigned char* m = (const unsigned char*)  mask;
  const unsigned char* n = (const unsigned char*)  name;
  const unsigned char* ma = (const unsigned char*) mask;
  const unsigned char* na = (const unsigned char*) name;
  int   wild  = 0;
  int   calls = 0;

  assert(mask != NULL);
  assert(name != NULL);

  if (!mask || !name)
    return 0;

  while (calls++ < MATCH_MAX_CALLS)
  {
    if (*m == '*')
    {
      /*
       * XXX - shouldn't need to spin here, the mask should have been
       * collapsed before match is called
       */
      while (*m == '*')
        m++;
      wild = 1;
      ma = m;
      na = n;
    }

    if (!*m)
    {
      if (!*n)
        return 1;
      for (m--; (m > (const unsigned char*) mask) && (*m == '?'); m--)
        ;
      if (*m == '*' && (m > (const unsigned char*) mask))
        return 1;
      if (!wild)
        return 0;
      m = ma;
      n = ++na;
    }
    else if (!*n)
    {
      /*
       * XXX - shouldn't need to spin here, the mask should have been
       * collapsed before match is called
       */
      while (*m == '*')
        m++;
      return (*m == 0);
    }
    if (ToLower(*m) != ToLower(*n) && *m != '?')
    {
      if (!wild)
        return 0;
      m = ma;
      n = ++na;
    }
    else
    {
      if (*m)
        m++;
      if (*n)
        n++;
    }
  }
  return 0;
}

/* match_esc()
 *
 * The match() function with support for escaping characters such
 * as '*' and '?'
 */
int match_esc(const char *mask, const char *name)
{
  const unsigned char* m = (const unsigned char*)  mask;
  const unsigned char* n = (const unsigned char*)  name;
  const unsigned char* ma = (const unsigned char*) mask;
  const unsigned char* na = (const unsigned char*) name;
  int   wild  = 0;
  int   calls = 0;
  int   quote = 0;

  assert(mask != NULL);
  assert(name != NULL);

  if (!mask || !name)
    return 0;

  while (calls++ < MATCH_MAX_CALLS)
  {
    if (quote)
      quote++;
    if (quote == 3)
      quote = 0;
    if (*m == '\\' && !quote)
    {
      m++;
      quote = 1;
      continue;
    }
    if (!quote && *m == '*')
    {
      /*
       * XXX - shouldn't need to spin here, the mask should have been
       * collapsed before match is called
       */
      while (*m == '*')
        m++;
      if (*m == '\\')
      {
	m++;
	/* This means it is an invalid mask -A1kmm. */
	if (!*m)
	  return 0;
	quote = 2;
      }
      wild = 1;
      ma = m;
      na = n;
    }

    if (!*m)
    {
      if (!*n)
        return 1;
      if (quote)
        return 0;
      for (m--; (m > (const unsigned char*) mask) && (*m == '?'); m--)
        ;
      if (*m == '*' && (m > (const unsigned char*) mask))
        return 1;
      if (!wild)
        return 0;
      m = ma;
      n = ++na;
    }
    else if (!*n)
    {
      /*
       * XXX - shouldn't need to spin here, the mask should have been
       * collapsed before match is called
       */
      if (quote)
        return 0;
      while (*m == '*')
        m++;
      return (*m == 0);
    }
    if (ToLower(*m) != ToLower(*n) && !(!quote && *m == '?')) {
      if (!wild)
        return 0;
      m = ma;
      n = ++na;
    }
    else
    {
      if (*m)
        m++;
      if (*n)
        n++;
    }
  }
  return 0;
}

/* collapse()
 *
 * collapses a string containing multiple *'s.
 */
char *
collapse(char *pattern)
{
  char *p = pattern, *po = pattern;
  char c;
  int f = 0;

  if (p == NULL)
    return NULL;
 
  while ((c = *p++))
  {
    if (c == '*')
    {
      if (!(f & 1))
        *po++ = '*';
      f |= 1;
    }
    else
    {
      *po++ = c;
      f &= ~1;
    }
  }
  *po++ = 0;

  return pattern;
}

/* collapse_esc()
 *
 * The collapse() function with support for escaping characters
 */
char *collapse_esc(char *pattern)
{
 char *p = pattern, *po = pattern;
 char c;
 int f = 0;

 if (p == NULL)
   return NULL;
 
 while ((c = *p++))
 {
   if (!(f & 2) && c == '*')
   {
     if (!(f & 1))
       *po++ = '*';
     f |= 1;
   }
   else if (!(f & 2) && c == '\\')
   {
     *po++ = '\\';
     f |= 2;
   }
   else
   {
     *po++ = c;
     f &= ~3;
   }
 }
 *po++ = 0;
 return pattern;
}

/*
 * irccmp - case insensitive comparison of two 0 terminated strings.
 *
 *      returns  0, if s1 equal to s2
 *              <0, if s1 lexicographically less than s2
 *              >0, if s1 lexicographically greater than s2
 */
int irccmp(const char *s1, const char *s2)
{
  const unsigned char* str1 = (const unsigned char*) s1;
  const unsigned char* str2 = (const unsigned char*) s2;
  int   res;

  assert(s1 != NULL);
  assert(s2 != NULL);

  while ((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
  {
    if (*str1 == '\0')
      return 0;
    str1++;
    str2++;
  }
  return (res);
}

int ircncmp(const char* s1, const char *s2, int n)
{
  const unsigned char* str1 = (const unsigned char*) s1;
  const unsigned char* str2 = (const unsigned char*) s2;
  int res;
  assert(s1 != NULL);
  assert(s2 != NULL);

  while ((res = ToUpper(*str1) - ToUpper(*str2)) == 0)
  {
    str1++; str2++; n--;
    if (n == 0 || (*str1 == '\0' && *str2 == '\0'))
      return 0;
  }
  return (res);
}

/* cp1251 tolower table -KZer-Za */
const unsigned char ToLowerTab[] = { 
  0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
  0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
  0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
  0x1e, 0x1f,
  ' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
  '*', '+', ',', '-', '.', '/',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  ':', ';', '<', '=', '>', '?',
  '@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  '_',
  '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  0x7f,
  0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
  0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
  0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
  0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xb8, 0xa9,
  0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
  0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
  0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
  0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
  0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
  0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff,
  0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
  0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
  0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

/* cp1251 tolower table -KZer-Za */
const unsigned char ToUpperTab[] = { 
  0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
  0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
  0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
  0x1e, 0x1f,
  ' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
  '*', '+', ',', '-', '.', '/',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  ':', ';', '<', '=', '>', '?',
  '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
  'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
  'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^',
  0x5f,
  '`', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
  'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
  'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^',
  0x7f,
  0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
  0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
  0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
  0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
  0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
  0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xa8, 0xb9,
  0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
  0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
  0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
  0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
  0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
  0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
  0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
  0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
  0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf
};

/*
 * CharAttrs table
 *
 * NOTE: RFC 1459 sez: anything but a ^G, comma, or space is allowed
 * for channel names
 */
const unsigned int CharAttrs[] = {
/* 0  */     CNTRL_C,
/* 1  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 2  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 3  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 4  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 5  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 6  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 7 BEL */  CNTRL_C|NONEOS_C,
/* 8  \b */  CNTRL_C|CHAN_C|NONEOS_C,
/* 9  \t */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 10 \n */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C|EOL_C,
/* 11 \v */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 12 \f */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 13 \r */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C|EOL_C,
/* 14 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 15 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 16 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 17 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 18 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 19 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 20 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 21 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 22 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 23 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 24 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 25 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 26 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 27 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 28 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 29 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 30 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 31 */     CNTRL_C|CHAN_C|NONEOS_C,
/* SP */     PRINT_C|SPACE_C,
/* ! */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C,
/* " */      PRINT_C|CHAN_C|NONEOS_C,
/* # */      PRINT_C|CHANPFX_C|CHAN_C|NONEOS_C,
/* $ */      PRINT_C|CHAN_C|NONEOS_C|USER_C,
/* % */      PRINT_C|CHAN_C|NONEOS_C,
/* & */      PRINT_C|CHANPFX_C|CHAN_C|NONEOS_C,
/* ' */      PRINT_C|CHAN_C|NONEOS_C,
/* ( */      PRINT_C|CHAN_C|NONEOS_C,
/* ) */      PRINT_C|CHAN_C|NONEOS_C,
/* * */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|SERV_C,
/* + */      PRINT_C|CHAN_C|NONEOS_C,
/* , */      PRINT_C|NONEOS_C,
/* - */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* . */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|USER_C|HOST_C|SERV_C,
/* / */      PRINT_C|CHAN_C|NONEOS_C,
/* 0 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 1 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 2 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 3 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 4 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 5 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 6 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 7 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 8 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* 9 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* : */      PRINT_C|CHAN_C|NONEOS_C|HOST_C,
/* ; */      PRINT_C|CHAN_C|NONEOS_C,
/* < */      PRINT_C|CHAN_C|NONEOS_C,
/* = */      PRINT_C|CHAN_C|NONEOS_C,
/* > */      PRINT_C|CHAN_C|NONEOS_C,
/* ? */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C,
/* @ */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C,
/* A */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* B */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* C */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* D */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* E */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* F */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* G */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* H */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* I */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* J */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* K */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* L */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* M */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* N */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* O */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* P */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* Q */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* R */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* S */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* T */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* U */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* V */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* W */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* X */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* Y */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* Z */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* [ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* \ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* ] */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* ^ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* _ */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* ` */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* a */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* b */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* c */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* d */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* e */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* f */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* g */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* h */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* i */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* j */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* k */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* l */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* m */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* n */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* o */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* p */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* q */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* r */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* s */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* t */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* u */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* v */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* w */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* x */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* y */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* z */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* { */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* | */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* } */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* ~ */      PRINT_C|ALPHA_C|CHAN_C|NONEOS_C|USER_C,
/* del  */   CHAN_C|NONEOS_C,
/* 0x80 */   CHAN_C|NONEOS_C,
/* 0x81 */   CHAN_C|NONEOS_C,
/* 0x82 */   CHAN_C|NONEOS_C,
/* 0x83 */   CHAN_C|NONEOS_C,
/* 0x84 */   CHAN_C|NONEOS_C,
/* 0x85 */   CHAN_C|NONEOS_C,
/* 0x86 */   CHAN_C|NONEOS_C,
/* 0x87 */   CHAN_C|NONEOS_C,
/* 0x88 */   CHAN_C|NONEOS_C,
/* 0x89 */   CHAN_C|NONEOS_C,
/* 0x8A */   CHAN_C|NONEOS_C,
/* 0x8B */   CHAN_C|NONEOS_C,
/* 0x8C */   CHAN_C|NONEOS_C,
/* 0x8D */   CHAN_C|NONEOS_C,
/* 0x8E */   CHAN_C|NONEOS_C,
/* 0x8F */   CHAN_C|NONEOS_C,
/* 0x90 */   CHAN_C|NONEOS_C,
/* 0x91 */   CHAN_C|NONEOS_C,
/* 0x92 */   CHAN_C|NONEOS_C,
/* 0x93 */   CHAN_C|NONEOS_C,
/* 0x94 */   CHAN_C|NONEOS_C,
/* 0x95 */   CHAN_C|NONEOS_C,
/* 0x96 */   CHAN_C|NONEOS_C,
/* 0x97 */   CHAN_C|NONEOS_C,
/* 0x98 */   CHAN_C|NONEOS_C,
/* 0x99 */   CHAN_C|NONEOS_C,
/* 0x9A */   CHAN_C|NONEOS_C,
/* 0x9B */   CHAN_C|NONEOS_C,
/* 0x9C */   CHAN_C|NONEOS_C,
/* 0x9D */   CHAN_C|NONEOS_C,
/* 0x9E */   CHAN_C|NONEOS_C,
/* 0x9F */   CHAN_C|NONEOS_C,
/* 0xA0 */   CHAN_C|NONEOS_C,
/* 0xA1 */   CHAN_C|NONEOS_C,
/* 0xA2 */   CHAN_C|NONEOS_C,
/* 0xA3 */   CHAN_C|NONEOS_C,
/* 0xA4 */   CHAN_C|NONEOS_C,
/* 0xA5 */   CHAN_C|NONEOS_C,
/* 0xA6 */   CHAN_C|NONEOS_C,
/* 0xA7 */   CHAN_C|NONEOS_C,
/* 0xA8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xA9 */   CHAN_C|NONEOS_C,
/* 0xAA */   CHAN_C|NONEOS_C,
/* 0xAB */   CHAN_C|NONEOS_C,
/* 0xAC */   CHAN_C|NONEOS_C,
/* 0xAD */   CHAN_C|NONEOS_C,
/* 0xAE */   CHAN_C|NONEOS_C,
/* 0xAF */   CHAN_C|NONEOS_C,
/* 0xB0 */   CHAN_C|NONEOS_C,
/* 0xB1 */   CHAN_C|NONEOS_C,
/* 0xB2 */   CHAN_C|NONEOS_C,
/* 0xB3 */   CHAN_C|NONEOS_C,
/* 0xB4 */   CHAN_C|NONEOS_C,
/* 0xB5 */   CHAN_C|NONEOS_C,
/* 0xB6 */   CHAN_C|NONEOS_C,
/* 0xB7 */   CHAN_C|NONEOS_C,
/* 0xB8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xB9 */   CHAN_C|NONEOS_C,
/* 0xBA */   CHAN_C|NONEOS_C,
/* 0xBB */   CHAN_C|NONEOS_C,
/* 0xBC */   CHAN_C|NONEOS_C,
/* 0xBD */   CHAN_C|NONEOS_C,
/* 0xBE */   CHAN_C|NONEOS_C,
/* 0xBF */   CHAN_C|NONEOS_C,

/*********************************************************************/
/* cyrillization for cp1251 by KZer-Za only cp1251 is supported now. */
/*********************************************************************/

/* 0xC0 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC1 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC2 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC3 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC4 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC5 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC6 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC7 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xC9 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCA */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCB */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCC */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCD */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCE */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xCF */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD0 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD1 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD2 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD3 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD4 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD5 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD6 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD7 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xD9 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDA */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDB */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDC */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDD */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDE */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xDF */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE0 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE1 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE2 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE3 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE4 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE5 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE6 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE7 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xE9 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xEA */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xEB */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xEC */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xED */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xEE */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xEF */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF0 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF1 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF2 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF3 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF4 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF5 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF6 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF7 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF8 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xF9 */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFA */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFB */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFC */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFD */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFE */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C,
/* 0xFF */   PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C
};

/*
	this table prevents similair nick usage
	checking BOBA against  gives a match and possible
	message from server what this nick is already used
	but the users still free to use any liters in their
	nicks or channel names

	- KZer-Za
	- this tiny patch also remove AWFUL problem of
	   nick stealing or shadowing

	   in this table, unique code associated with zero
	   similair code associated with nonzero
*/

unsigned const char csim_table[] = {
/* 0  */		0,
/* 1  */		0,
/* 2  */		0,
/* 3  */		0,
/* 4  */		0,
/* 5  */		0,
/* 6  */		0,
/* 7 BEL */		0,
/* 8  \b */		0,
/* 9  \t */		0,
/* 10 \n */		0,
/* 11 \v */		0,
/* 12 \f */		0,
/* 13 \r */		0,
/* 14 */		0,
/* 15 */		0,
/* 16 */		0,
/* 17 */		0,
/* 18 */		0,
/* 19 */		0,
/* 20 */		0,
/* 21 */		0,
/* 22 */		0,
/* 23 */		0,
/* 24 */		0,
/* 25 */		0,
/* 26 */		0,
/* 27 */		0,
/* 28 */		0,
/* 29 */		0,
/* 30 */		0,
/* 31 */		0,
/* SP */		0,
/* ! */ 		0,
/* " */ 		0,
/* # */ 		0,
/* $ */ 		0,
/* % */ 		0,
/* & */ 		0,
/* ' */ 		0,
/* ( */ 		0,
/* ) */ 		0,
/* * */ 		0,
/* + */ 		0,
/* , */ 		0,
/* - */ 		0,
/* . */ 		0,
/* / */ 		0,
/* 0 */ 		0,
/* 1 */ 		0,
/* 2 */ 		0,
/* 3 */ 		0,
/* 4 */ 		0,
/* 5 */ 		0,
/* 6 */ 		0,
/* 7 */ 		0,
/* 8 */ 		0,
/* 9 */ 		0,
/* : */ 		0,
/* ; */ 		0,
/* < */ 		0,
/* = */ 		0,
/* > */ 		0,
/* ? */ 		0,
/* @ */			0,
/* A */			0,
/* B */			0,
/* C */			0,
/* D */			0,
/* E */			0,
/* F */			0,
/* G */			0,
/* H */			0,
/* I */			0,
/* J */			0,
/* K */			0,
/* L */			0,
/* M */			0,
/* N */			0,
/* O */			0,
/* P */			0,
/* Q */			0,
/* R */			0,
/* S */			0,
/* T */			0,
/* U */			0,
/* V */			0,
/* W */			0,
/* X */			0,
/* Y */			0,
/* Z */			0,
/* [ */			0,
/* \ */			0,
/* ] */			0,
/* ^ */			0,
/* _ */			0,
/* ` */			0,
/* a */			0,
/* b */			0,
/* c */			0,
/* d */			0,
/* e */			0,
/* f */			0,
/* g */			0,
/* h */			0,
/* i */			0,
/* j */			0,
/* k */			0,
/* l */			0,
/* m */			0,
/* n */			0,
/* o */			0,
/* p */			0,
/* q */			0,
/* r */			0,
/* s */			0,
/* t */			0,
/* u */			0,
/* v */			0,
/* w */			0,
/* x */			0,
/* y */			0,
/* z */			0,
/* { */			0,
/* | */			0,
/* } */			0,
/* ~ */			0,
/* del  */			0,
/* 0x80 */			0,
/* 0x81 */			0,
/* 0x82 */			0,
/* 0x83 */			0,
/* 0x84 */			0,
/* 0x85 */			0,
/* 0x86 */			0,
/* 0x87 */			0,
/* 0x88 */			0,
/* 0x89 */			0,
/* 0x8A */			0,
/* 0x8B */			0,
/* 0x8C */			0,
/* 0x8D */			0,
/* 0x8E */			0,
/* 0x8F */			0,
/* 0x90 */			0,
/* 0x91 */			0,
/* 0x92 */			0,
/* 0x93 */			0,
/* 0x94 */			0,
/* 0x95 */			0,
/* 0x96 */			0,
/* 0x97 */			0,
/* 0x98 */			0,
/* 0x99 */			0,
/* 0x9A */			0,
/* 0x9B */			0,
/* 0x9C */			0,
/* 0x9D */			0,
/* 0x9E */			0,
/* 0x9F */			0,
/* 0xA0 */			0,
/* 0xA1 */			0,
/* 0xA2 */			0,
/* 0xA3 */			0,
/* 0xA4 */			0,
/* 0xA5 */			0,
/* 0xA6 */			0,
/* 0xA7 */			0,
/* 0xA8 */			0,
/* 0xA9 */			0,
/* 0xAA */			0,
/* 0xAB */			0,
/* 0xAC */			0,
/* 0xAD */			0,
/* 0xAE */			0,
/* 0xAF */			0,
/* 0xB0 */			0,
/* 0xB1 */			0,
/* 0xB2 */			0,
/* 0xB3 */			0,
/* 0xB4 */			0,
/* 0xB5 */			0,
/* 0xB6 */			0,
/* 0xB7 */			0,
/* 0xB8 */			0,
/* 0xB9 */			0,
/* 0xBA */			0,
/* 0xBB */			0,
/* 0xBC */			0,
/* 0xBD */			0,
/* 0xBE */			0,
/* 0xBF */			0,
/* 0xC0 */			0,
/* 0xC1 */			0,
/* 0xC2 */			0,
/* 0xC3 */			0,
/* 0xC4 */			0,
/* 0xC5 */			0,
/* 0xC6 */			0,
/* 0xC7 */			0,
/* 0xC8 */			0,
/* 0xC9 */			0,
/* 0xCA */			0,
/* 0xCB */			0,
/* 0xCC */			0,
/* 0xCD */			0,
/* 0xCE */			0,
/* 0xCF */			0,
/* 0xD0 */			0,
/* 0xD1 */			0,
/* 0xD2 */			0,
/* 0xD3 */			0,
/* 0xD4 */			0,
/* 0xD5 */			0,
/* 0xD6 */			0,
/* 0xD7 */			0,
/* 0xD8 */			0,
/* 0xD9 */			0,
/* 0xDA */			0,
/* 0xDB */			0,
/* 0xDC */			0,
/* 0xDD */			0,
/* 0xDE */			0,
/* 0xDF */			0,
/* 0xE0 */			0,
/* 0xE1 */			0,
/* 0xE2 */			0,
/* 0xE3 */			0,
/* 0xE4 */			0,
/* 0xE5 */			0,
/* 0xE6 */			0,
/* 0xE7 */			0,
/* 0xE8 */			0,
/* 0xE9 */			0,
/* 0xEA */			0,
/* 0xEB */			0,
/* 0xEC */			0,
/* 0xED */			0,
/* 0xEE */			0,
/* 0xEF */			0,
/* 0xF0 */			0,
/* 0xF1 */			0,
/* 0xF2 */			0,
/* 0xF3 */			0,
/* 0xF4 */			0,
/* 0xF5 */			0,
/* 0xF6 */			0,
/* 0xF7 */			0,
/* 0xF8 */			0,
/* 0xF9 */			0,
/* 0xFA */			0,
/* 0xFB */			0,
/* 0xFC */			0,
/* 0xFD */			0,
/* 0xFE */			0,
/* 0xFF */			0,
};